package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesWorkOrderDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.BeanUtil;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.MesWorkOrderModel;
import lombok.extern.slf4j.Slf4j;
import org.hibernate.SQLQuery;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @Description :SAP生产计划
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesWorkOrderDaoImpl implements IMesWorkOrderDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public List<MesWorkOrderModel> sumMesWorkOrderQtyToSap(String organizeCode) {
        StringBuffer hql = new StringBuffer("select organize_code as organizeCode,part_no as partNo, date_format(plan_start_time, '%Y-%m-%d' ) as planStartTime, product_version as productVersion, " +
                " sum(case when work_order_status = 40 then complete_qty else qty end ) as qty  " +
                " from mes_work_order where is_deleted=:isDeleted and is_valid = :isValid and organize_code = :organizeCode " +
                " and work_order_type = 20 and order_flag !='P' and work_order_status != 10 and plan_start_time >= :today" +
                " group by organize_code,part_no, date_format(plan_start_time, '%Y-%m-%d' ), product_version ");
        Query query = entityManager.createNativeQuery(hql.toString());
        query.setParameter("organizeCode", organizeCode);
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("today", TimeTool.getToday());
        query.unwrap(SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);

        List<HashMap<String, Object>> hashMaps = query.getResultList();
        List<MesWorkOrderModel> mesWorkOrderList =new ArrayList<>();
        for (HashMap<String, Object> hashMap : hashMaps) {
            try {
                mesWorkOrderList.add((MesWorkOrderModel) BeanUtil.populateBean(hashMap, MesWorkOrderModel.class));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return mesWorkOrderList;
    }
}
