package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesProductionRecordService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesProdRuleNosortCfgDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.BeanUtil;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRuleNosortCfg;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionRecord;
import cn.estsh.i3plus.pojo.mes.model.MesProductionRecordModel;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.hibernate.SQLQuery;
import org.hibernate.transform.Transformers;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 设备加工记录表
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/28 9:27
 * @Modify:
 **/
@Service
@Slf4j
public class MesProductionRecordServiceImpl extends BaseMesService<MesProductionRecord> implements IMesProductionRecordService {
    @Autowired
    private EntityManager entityManager;

    @Autowired
    private IMesProdRuleNosortCfgDao mesProdRuleNosortCfgDao;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    @Override
    public List<MesProductionRecord> findMesProductionRecord(String organizeCode, String sn) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(sn, "productSn", packBean);
        DdlPreparedPack.getOrderBy("createDatetime", CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), packBean);
        List<MesProductionRecord> produceSnList = baseRDao.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(produceSnList)) {
            MesException.throwMesBusiException("条码【%s】加工记录信息不存在", sn);
        }
        return produceSnList;
    }

    @Override
    public ListPager queryRecordGroupByPartNoWorkCellCode(MesProductionRecord mesProductionRecord, Pager pager) {

        return queryERPMaterialStockHqlPager(mesProductionRecord, pager);
    }

    @Override
    public ListPager<MesProductionRecordModel> queryMesProductionOffLine(MesProductionRecordModel mesProductionRecordModel, Pager pager) {
        List<MesProdRuleNosortCfg> mesProdRuleNosortCfgs = mesProdRuleNosortCfgDao.findMesProdRuleNosortCfgByWorkCellCode(mesProductionRecordModel.getOnLineWorkCellCode(), mesProductionRecordModel.getPartNo(), mesProductionRecordModel.getOrganizeCode());
        if(CollectionUtils.isEmpty(mesProdRuleNosortCfgs) || StringUtil.isEmpty(mesProdRuleNosortCfgs.iterator().next().getOutPartNo())) return new ListPager<>(new ArrayList<>(),pager);
        //筛选下线工位的当前零件的产成条码
        List<MesProduceSn> mesProduceSnList = getMesProduceSnList(mesProductionRecordModel);
        if(CollectionUtils.isEmpty(mesProduceSnList)) return new ListPager<>(new ArrayList<>(),pager);
        //在上线工位消耗进料条码且没有报工
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesProductionRecordModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProdRuleNosortCfgs.iterator().next().getOutPartNo(),"partNo",packBean);
        DdlPreparedPack.getStringEqualPack(mesProductionRecordModel.getOnLineWorkCellCode(),"workCellCode",packBean);
        DdlPreparedPack.getInPackList(mesProduceSnList.stream().map(MesProduceSn::getProductSn).distinct().collect(Collectors.toList()),"productSn",packBean);
        List<MesProductionRecord> productionRecordList = baseRDao.findByHqlWhere(packBean);
        Map<String,MesProductionRecord> productionRecordMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(productionRecordList)){
            for (MesProductionRecord productionRecord : productionRecordList) {
                productionRecordMap.put(productionRecord.getProductSn(),productionRecord);
            }
        }
        //数据组装
        List<MesProductionRecordModel> mesProductionRecordModelList = new ArrayList<>();
        MesProductionRecordModel model;
        for (MesProduceSn mesProduceSn : mesProduceSnList) {
            model = new MesProductionRecordModel();
            BeanUtils.copyProperties(mesProduceSn,model);
            if(!productionRecordMap.containsKey(mesProduceSn.getProductSn()) || (productionRecordMap.containsKey(mesProduceSn.getProductSn()) && MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue() == productionRecordMap.get(mesProduceSn.getProductSn()).getReportStatus())){
                model.setOffLineWorkCellCode(mesProduceSn.getWorkCellCode());
                mesProductionRecordModelList.add(model);
            }
        }
        pager = PagerHelper.getPager(pager, mesProductionRecordModelList.size());
        return new ListPager<>(mesProductionRecordModelList.subList(pager.getStartRow(),
                Math.min(pager.getPageSize() * pager.getCurrentPage(), mesProductionRecordModelList.size())), pager);
    }

    private List<MesProduceSn> getMesProduceSnList(MesProductionRecordModel mesProductionRecordModel) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesProductionRecordModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProductionRecordModel.getPartNo(),"partNo",packBean);
        DdlPreparedPack.getStringEqualPack(mesProductionRecordModel.getOffLineWorkCellCode(),"workCellCode",packBean);
        DdlPreparedPack.getStringBiggerPack(mesProductionRecordModel.getCreateDateTimeStart(),"createDatetime",packBean);
        DdlPreparedPack.getStringSmallerPack(mesProductionRecordModel.getCreateDateTimeEnd(),"createDatetime",packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue(),"snStatus",packBean);
        return mesProduceSnRepository.findByHqlWhere(packBean);
    }

    private ListPager<MesProductionRecord> queryERPMaterialStockHqlPager(MesProductionRecord mesProductionRecord, Pager pager) {
        String org = mesProductionRecord.getOrganizeCode();
        String startTime = mesProductionRecord.getCompleteDateTimeStart();
        String endTime = mesProductionRecord.getCompleteDateTimeEnd();
        List<MesProductionRecord> mesProductionRecordList = new ArrayList<>();
        String countSql = "select count(1)  ";
        String commHql = " SELECT " +
                " DISTINCT mpr.id as id," +
                " mpr.organize_code as organizeCode," +
                " mpr.create_date_time AS createDatetime, " +
                " mpr.create_user AS createUser, " +
                " mpr.modify_date_time AS  modifyDatetime, " +
                " mpr.modify_user AS  modifyUser, " +
                " mpr.part_no as partNo," +
                " mpr.part_name as partName," +
                " mpr.work_center_code as workCenterCode," +
                " mpr.shift_code as shiftCode," +
                " mpr.work_cell_code as workCellCode," +
                " mpr.equipment_name as equipmentName," +
                " mpr.complete_date_time as completeDateTime," +
                " count( mpr.qty) as qty ";

        String sql = " from mes_production_record mpr" +
                " where 1=1  " +
                " and mpr.organize_code =:organizeCode " +
                " and mpr.report_type !='10' " +
                " and mpr.is_deleted ='2' " +
                " and mpr.is_valid ='1' ";
        if (!StringUtil.isEmpty(startTime)) {
            sql += " and mpr.complete_date_time >=:startTime ";
        }
        if (!StringUtil.isEmpty(endTime)) {
            sql += " and mpr.complete_date_time <:endTime ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getPartNo())) {
            sql += " and mpr.part_no ='" + mesProductionRecord.getPartNo() + "' ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getWorkCenterCode())) {
            sql += " and mpr.work_center_code ='" + mesProductionRecord.getWorkCenterCode() + "' ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getShiftCode())) {
            sql += " and mpr.shift_code ='" + mesProductionRecord.getShiftCode() + "' ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getWorkCellCode())) {
            sql += " and mpr.work_cell_code ='" + mesProductionRecord.getWorkCellCode() + "' ";
        }

        if (!StringUtil.isEmpty(mesProductionRecord.getEquipmentName())) {
            sql += " and mpr.equipment_name like '%" + mesProductionRecord.getEquipmentName() + "%' ";
        }

        sql += " GROUP BY mpr.part_no,mpr.work_cell_code ORDER BY mpr.complete_date_time desc ";

        Query queryObject = entityManager.createNativeQuery(commHql + sql);
        queryObject.setParameter("organizeCode", org);
        if (!StringUtil.isEmpty(startTime)) {
            queryObject.setParameter("startTime", startTime);
        }
        if (!StringUtil.isEmpty(endTime)) {
            queryObject.setParameter("endTime", endTime);
        }

        Query queryCountObject = entityManager.createNativeQuery(countSql + sql);
        queryCountObject.setParameter("organizeCode", org);
        if (!StringUtil.isEmpty(startTime)) {
            queryCountObject.setParameter("startTime", startTime);
        }
        if (!StringUtil.isEmpty(endTime)) {
            queryCountObject.setParameter("endTime", endTime);
        }

        queryObject.unwrap(SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP)
                .setMaxResults(pager.getPageSize()).setFirstResult(pager.getStartRow()).getResultList();
        queryCountObject.unwrap(SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);

        List resultList = queryCountObject.getResultList();
        int countSize = resultList.size();
        if (countSize <= 0) {
            return new ListPager(mesProductionRecordList,pager);
        }

        pager = PagerHelper.getPager(pager, countSize);

        List<HashMap<String, Object>> hashMaps = queryObject.getResultList();

        for (HashMap<String, Object> hashMap : hashMaps) {
            try {
                mesProductionRecordList.add((MesProductionRecord) BeanUtil.populateBean(hashMap, MesProductionRecord.class));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return new ListPager(mesProductionRecordList, pager);

    }

}
